#include <stdio.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <net/if.h>
#include <arpa/inet.h>
#include <signal.h>
#include <string.h>

#include "ddnsd-conf.h"
#include "ddnsd-update.h"

/* Time to wait in seconds between checking the interfaces */
#define TIME_WAIT 10

FILE *status = NULL;

int time_to_quit = 0;

void quit_handler(int signal )
{
  time_to_quit = 1;
}


int get_if_addr(int sock, char *name, struct sockaddr_in *sin)
{
  struct ifreq ifr;

  memset(&ifr, 0, sizeof(ifr));
  strcpy(ifr.ifr_name, name);
  /* why does this need to be done twice? */
  if(ioctl(sock, SIOCGIFADDR, &ifr) < 0)
  {
    /* Don't print out error because the interface might not be up yet */
    // perror("ioctl(SIOCGIFADDR)");
    memset(sin, 0, sizeof(struct sockaddr_in));
    return -1;
  }
  if(ioctl(sock, SIOCGIFADDR, &ifr) < 0)
  {
    perror("ioctl(SIOCGIFADDR)");
    memset(sin, 0, sizeof(struct sockaddr_in));
    return -1;
  }
  if(ifr.ifr_addr.sa_family == AF_INET)
  {
    memcpy(sin, &(ifr.ifr_addr), sizeof(struct sockaddr_in));
    return 0;
  }
  else
  {
    memset(sin, 0, sizeof(struct sockaddr_in));
    return -1;
  }
  return -1;
}

void update_hostname_addrs( char *hostname ) {

}

void print_hostname_addrs( char *hostname ) {
  char *interface, *cache_ip;
  struct sockaddr_in sin;
  char *service, *address, *username, *password;
  int retval, update = 0;
  int sock = socket( AF_INET, SOCK_STREAM, 0 );

  interface = BcmDDnsConfig_getInterface( hostname );

  if (get_if_addr( sock, interface, &sin ) == 0 ) {
    if ( ( ( cache_ip = BcmDDnsCache_get( hostname ) ) != NULL ) &&
         !strcmp( cache_ip, inet_ntoa( sin.sin_addr ) ) ) {
      // P.T. clean up resource
      close( sock );
      return;
    }
    address = strdup( inet_ntoa( sin.sin_addr ) );
//#ifdef BRCM_DEBUG
    printf( "ddnsd: %s (%s) IP has changed from %s to %s\n", hostname, interface, cache_ip, address );
//#endif
    service = BcmDDnsConfig_getService( hostname );
    username = BcmDDnsConfig_getUsername( hostname );
    password = BcmDDnsConfig_getPassword( hostname );
    printf("service: %s, username: %s, password: %s\n", service, username, password);
    retval = do_update( service, hostname, address, username, password );
    switch( retval ) {
      case UPDATERES_OK:
        BcmDDnsCache_update( hostname, inet_ntoa( sin.sin_addr ) );
        update = 1;
        break;
      case UPDATERES_SHUTDOWN:
      case UPDATERES_BADSERVICE:
        BcmDDnsConfig_remove( hostname );
        break;
      case UPDATERES_ERROR:
        break;
    }
    free( address );
  }
  if( update ) {
    BcmDDnsCache_save( "/var/ddnsd.cache" );
  }
  close( sock );
}

//#ifdef BUILD_STATIC
//int ddnsd_main(int argc, char **argv) {
//#else
int main(int argc, char **argv ) {
//#endif	
  struct ddns_cache *cache;
  struct sockaddr_in sin;
  int i, sock;

  if( BcmDDnsConfig_init( argv[1] ) == -1 ) {
//#ifdef BRCM_DEBUG
    printf("ddnsd: No interfaces defined.\n");
//#endif
    exit(1);
  }
  	
  BcmDDnsCache_init( argv[2] );

  signal( SIGHUP, &quit_handler );
  signal( SIGTERM, &quit_handler );
  signal( SIGQUIT, &quit_handler );

  for(;!time_to_quit;) {
    BcmDDnsConfig_iter( print_hostname_addrs );
    sleep(TIME_WAIT);
  }
  BcmDDnsConfig_close();
  BcmDDnsCache_save( argv[2] );
//#ifdef BRCM_DEBUG
  printf( "ddnsd: Exiting...\n");
//#endif
}
